#ifndef TREE_H
#define TREE_H

/*
 *  tree.h
 *  Mothur
 *
 *  Created by Sarah Westcott on 1/22/09.
 *  Copyright 2009 Schloss Lab UMASS Amherst. All rights reserved.
 *
 */

#include "treenode.h"
#include "treemap.h"
/* This class represents the treefile. */

class Tree {
public: 
	Tree(string);  //do not use tree generated by this constructor its just to extract the treenames, its a chicken before the egg thing that needs to be revisited.
	Tree(int, TreeMap*); 
	Tree(TreeMap*);		//to generate a tree from a file
    Tree(TreeMap*, vector< vector<double> >&); //create tree from sim matrix
	~Tree();
	
    TreeMap* getTreeMap() { return tmap; }
	void getCopy(Tree*);  //makes tree a copy of the one passed in.
    void getCopy(Tree* copy, map<string, string>); //makes a copy of the tree structure passed in, (just parents, children and br). Used with the Tree(TreeMap*) constructor. Assumes the tmap already has set seqs groups you want.  Used by subsample to reassign seqs you don't want included to group "doNotIncludeMe".
	void getSubTree(Tree*, vector<string>);  //makes tree a that contains only the names passed in.
    int getSubTree(Tree* originalToCopy, vector<string> seqToInclude, map<string, string> nameMap);  //used with (int, TreeMap) constructor. SeqsToInclude contains subsample wanted - assumes these are unique seqs and size of vector=numLeaves passed into constructor. nameMap is unique -> redundantList can be empty if no namesfile was provided. 
    
	void assembleRandomTree();
	void assembleRandomUnifracTree(vector<string>);
	void assembleRandomUnifracTree(string, string);
    
	void createNewickFile(string);
	int getIndex(string);
	void setIndex(string, int);
	int getNumNodes() { return numNodes; }
	int getNumLeaves(){	return numLeaves; }
	map<string, int> mergeUserGroups(int, vector<string>);  //returns a map with a groupname and the number of times that group was seen in the children
	void printTree();
	void print(ostream&);
	void print(ostream&, string);
    void print(ostream&, map<string, string>);
	int findRoot();  //return index of root node
	
	//this function takes the leaf info and populates the non leaf nodes
	int assembleTree(map<string, string>);	
	
	vector<Node> tree;		//the first n nodes are the leaves, where n is the number of sequences.
	map< string, vector<int> > groupNodeInfo;	//maps group to indexes of leaf nodes with that group, different groups may contain same node because of names file.
			
private:
	TreeMap* tmap;
	int numNodes, numLeaves;
	ofstream out;
	string filename;
	
    map<string, string> names;
	map<string, int>::iterator it, it2;
	map<string, int> mergeGroups(int);  //returns a map with a groupname and the number of times that group was seen in the children
	map<string,int> mergeGcounts(int);
	
	void addNamesToCounts(map<string, string>);
	void randomTopology();
	void randomBlengths();
	void randomLabels(vector<string>);
	//void randomLabels(string, string);
	void printBranch(int, ostream&, map<string, string>);  //recursively print out tree
    void printBranch(int, ostream&, string);
	void parseTreeFile();	//parses through tree file to find names of nodes and number of them
							//this is required in case user has sequences in the names file that are
							//not included in the tree. 
							//only takes names from the first tree in the tree file and assumes that all trees use the same names.
	int readTreeString(ifstream&);
	int populateNewTree(vector<Node>&, int, int&);
	void printBranch(int, ostream&, string, vector<Node>&);
		
	MothurOut* m;
		
};

#endif
